!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
!  n :    The N-body problem
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
   SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
!  ---------- ---- 

   IMPLICIT NONE
   INTEGER, INTENT(IN) :: NDIM, ICP(*), IJAC
   DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
   DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
   DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,NDIM), DFDP(NDIM,*)

   INTEGER N, ND, i ,j, k
   PARAMETER (N=3, ND=3)
   DOUBLE PRECISION d, x(N,ND), v(N,ND), xp(N,ND), vp(N,ND), dist

     DO i=1,N
       DO k=1,ND
         x(i,k) = U((i-1)*ND+k) 
         v(i,k) = U((N+i-1)*ND+k) 
       ENDDO            
     ENDDO            

     DO i=1,N
       DO k=1,ND
          xp(i,k) = v(i,k) 
          vp(i,k) = PAR(22+k) + PAR(22)*v(i,k) + PAR(29)*x(i,k) 
       ENDDO 
 
       DO j=1,N
         IF (j.NE.i) THEN 
           d = dist(x,i,j)**3
           DO k=1,ND
             vp(i,k) = vp(i,k) - PAR(j)*(x(i,k)-x(j,k))/d 
           ENDDO 
         ENDIF
       ENDDO
       vp(i,1) = vp(i,1)                  - PAR(27)*x(i,3) + PAR(28)*x(i,2) 
       vp(i,2) = vp(i,2) + PAR(26)*x(i,3)                  - PAR(28)*x(i,1) 
       vp(i,3) = vp(i,3) - PAR(26)*x(i,2) + PAR(27)*x(i,1) 
     ENDDO 

     DO i=1,N
       DO k=1,ND
         F((i-1)*ND+k) = PAR(11)*xp(i,k) 
         F((i+2)*ND+k) = PAR(11)*vp(i,k) 
       ENDDO
     ENDDO

   END SUBROUTINE FUNC
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
   SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC) 
!  ---------- ---- 

   IMPLICIT NONE
   INTEGER, INTENT(IN) :: NDIM, ICP(*), NBC, IJAC
   DOUBLE PRECISION, INTENT(IN) :: PAR(*), U0(NDIM), U1(NDIM)
   DOUBLE PRECISION, INTENT(OUT) :: FB(NBC)
   DOUBLE PRECISION, INTENT(INOUT) :: DBC(NBC,*)

   DOUBLE PRECISION dist, normsq
   INTEGER N, ND, i ,j, k
   PARAMETER (N=3, ND=3)
   DOUBLE PRECISION x(N,ND), v(N,ND), KE, PE, TE, A

     DO i=1,NDIM
       FB(i) = U0(i) - U1(i) 
     ENDDO

     DO i=1,N
       DO k=1,ND
         x(i,k) = U0((i-1)*ND+k) 
         v(i,k) = U0((N+i-1)*ND+k) 
       ENDDO
     ENDDO

     KE = 0.d0
     DO i=1,N
       normsq = 0.d0
       DO k=1,ND
         normsq = normsq + v(i,k)**2
       ENDDO
       KE = KE +  0.5d0*PAR(i)*normsq
     ENDDO

     PE = 0.d0
     DO i=1,N
       DO j=i+1,N
         PE = PE - PAR(i)*PAR(j)/dist(x,i,j) 
       ENDDO
     ENDDO
     TE = PE + KE 
     FB(NDIM+1) = TE - PAR(4) 

     A=0.d0 
     DO i=22,29
       A = A + PAR(i)**2
     ENDDO
     FB(NDIM+2) = DSQRT(A) - PAR(5) 

   END SUBROUTINE BCND
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
   SUBROUTINE ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,FI,IJAC,DINT) 
!  ---------- ---- 

   IMPLICIT NONE
   INTEGER, INTENT(IN) :: NDIM, ICP(*), NINT, IJAC
   DOUBLE PRECISION, INTENT(IN) :: PAR(*)
   DOUBLE PRECISION, INTENT(IN) :: U(NDIM), UOLD(NDIM), UDOT(NDIM), UPOLD(NDIM)
   DOUBLE PRECISION, INTENT(OUT) :: FI(NINT)
   DOUBLE PRECISION, INTENT(INOUT) :: DINT(NINT,*)
   INTEGER N, ND, i, i1, i2, i3, j, k
   PARAMETER (N=3, ND=3)

     DO i=1,NINT
       FI(i) = 0.d0
     ENDDO

     DO i=1,ND*N
 	FI(1) = FI(1) + U(i)*UPOLD(i)
     ENDDO

     DO i=1,N
       i1=(i-1)*ND+1
       i2=(i-1)*ND+2
       i3=(i-1)*ND+3
       FI(2) = FI(2) + U(i1) - UOLD(i1) 
       FI(3) = FI(3) + U(i2) - UOLD(i2) 
       FI(4) = FI(4) + U(i3) - UOLD(i3) 
       FI(5) = FI(5) + (U(i2)- UOLD(i2))*U(i3) - (U(i3)-UOLD(i3))*U(i2) 
       FI(6) = FI(6) - (U(i1)- UOLD(i1))*U(i3) + (U(i3)-UOLD(i3))*U(i1) 
       FI(7) = FI(7) + (U(i1)- UOLD(i1))*U(i2) - (U(i2)-UOLD(i2))*U(i1) 
     ENDDO
    
   END SUBROUTINE ICND
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
   SUBROUTINE STPNT(NDIM,U,PAR,T)
!  ---------- -----

   IMPLICIT NONE
   INTEGER, INTENT(IN) :: NDIM
   DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM), PAR(*)
   DOUBLE PRECISION, INTENT(IN) :: T
   INTEGER i

     DO i=1,36
       PAR(i)=0.d0
     ENDDO

! Set the masses of the starting figure-8 orbit in cms.dat:
     PAR(1) = 1.d0
     PAR(2) = 1.d0
     PAR(3) = 1.d0

! Set the total energy of the figure-8 orbit in cms.dat:
     PAR(4)  =-1.2992240155d0	

! Set the actual period of the figure-8 orbit in cms.dat:
     PAR(11) = 6.2378785252d0

   END SUBROUTINE STPNT
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
   DOUBLE PRECISION FUNCTION dist(x,i,j) 

   INTEGER N, ND, i, j, k
   PARAMETER (N=3, ND=3)
   DOUBLE PRECISION x(N,ND)
      
     dist=0.d0 
     DO k=1,ND
       dist = dist + (x(i,k)-x(j,k))**2 
     ENDDO
     dist=DSQRT(dist) 

   RETURN
   END
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
   SUBROUTINE FOPT 
   END SUBROUTINE FOPT

   SUBROUTINE PVLS
   END SUBROUTINE PVLS
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
